import type HasFieldData from '../Interfaces/HasFieldData';
import type HasConditionData from "../Interfaces/HasConditionData";
import type HasHelpData from '../Interfaces/HasHelpData';
import type Fieldset from './Fieldset';
import type HasChoicesData from '../Interfaces/HasChoicesData';
import type HasValidationData from '../Interfaces/HasValidationData';
import CSSElement from './CSSElement';
/**
 * Field class.
 *
 * @since 1.0.0
 */
export default class Field extends CSSElement implements HasFieldData {
    readonly fieldset: Fieldset;
    readonly fields: Field[];
    readonly name: string;
    readonly type: string;
    readonly label: string;
    readonly prefix: string;
    readonly suffix: string;
    readonly placeholder: string;
    readonly multicol: number;
    readonly required: boolean;
    readonly defaultValue: any[];
    readonly dynamicValue: any[];
    readonly params: any[];
    readonly help: HasHelpData;
    readonly choices: HasChoicesData[];
    readonly conditions: HasConditionData[];
    readonly validations: HasValidationData[];
    value: any;
    private validated;
    private errors;
    private inputClasses;
    /**
     * Initializing field.
     *
     * @param name  Name of the field.
     * @param field Field object
     *
     * @since 1.0.0
     */
    constructor(fieldset: Fieldset, field: HasFieldData);
    /**
     * Set value of field.
     *
     * @param value Value to set.
     *
     * @since 1.0.0
     */
    setValue(value: any): void;
    /**
     * Get value of field.
     *
     * @return value Value to set.
     *
     * @since 1.0.0
     */
    getValue(): any;
    /**
     * Is there a prefix?
     *
     * @returns True if field has prefix.
     *
     * @since 1.0.0
     */
    hasPrefix(): boolean;
    /**
     * Get prefix.
     *
     * @returns Prefix.
     *
     * @since 1.0.0
     */
    getPrefix(): string;
    /**
     * Get suffix.
     *
     * @returns Suffix.
     *
     * @since 1.0.0
     */
    getSuffix(): string;
    /**
     * Is there a suffix?
     *
     * @returns True if field has suffix.
     *
     * @since 1.0.0
     */
    hasSuffix(): boolean;
    /**
     * Get default value.
     *
     * @return value Default value.
     *
     * @since 1.0.0
     */
    getDefaultValue(): any;
    /**
     * Get default value.
     *
     * @return value Default value.
     *
     * @since 1.0.0
     */
    getDynamicValue(): any;
    /**
     * Sets autovalues for default or dynamic value if exists.
     *
     * @since 1.0.0
     */
    autoValue(): void;
    /**
     * Does field have choices.
     *
     * @return True if it has choices, false if not.
     *
     * @since 1.0.0
     */
    hasChoices(): boolean;
    /**
     * Validate the field.
     *
     * @return Array of errors, empty array on no errors.
     *
     * @since 1.0.0
     */
    validate(): string[];
    /**
     * Get validation errors.
     *
     * @return Erros which occured while validating.
     *
     * @since 1.0.0
     */
    getValidationErors(): string[];
    /**
     * Has field validation errors.
     *
     * @return True if field has errors, false if not.
     *
     * @since 1.0.0
     */
    hasValidationErrors(): boolean;
    /**
     * Has the field already ben validated
     *
     * @return True if field has been validated, false if not.
     *
     * @since 1.0.0
     */
    hasBeenValidated(): boolean;
    /**
     * Checks if conditions are fullfilled.
     *
     * @returns True if conditions are fullfilled, false if not.
     *
     * @since 1.0.0
     */
    conditionsFullfilled(): boolean;
    /**
     * Add input class.
     *
     * @param string Name of css class to add.
     *
     * @since 1.0.0
     */
    addInputClass(name: string): void;
    /**
     * Remove input class.
     *
     * @param string Name of css class to remove.
     *
     * @since 1.0.0
     */
    removeInputClass(name: string): void;
    /**
     * Get input classes.
     *
     * @return String of CSS classes.
     *
     * @since 1.0.0
     */
    getInputClasses(): string;
}
