import Help from './Help';
import Validator from './Validator';
import DynamicValue from './DynamicValue';
import CSSElement from './CSSElement';
var startedMulticol = false;
/**
 * Field class.
 *
 * @since 1.0.0
 */
export default class Field extends CSSElement {
    fieldset;
    fields = [];
    name;
    type;
    label;
    prefix;
    suffix;
    placeholder;
    multicol;
    required;
    defaultValue;
    dynamicValue;
    params;
    help;
    choices;
    conditions;
    validations;
    value;
    validated = false;
    errors = [];
    inputClasses = [];
    /**
     * Initializing field.
     *
     * @param name  Name of the field.
     * @param field Field object
     *
     * @since 1.0.0
     */
    constructor(fieldset, field) {
        super();
        this.fieldset = fieldset;
        this.name = field.name;
        this.type = field.type;
        this.label = field.label;
        this.prefix = field.prefix;
        this.suffix = field.suffix;
        this.multicol = field.multicol === undefined ? 0 : field.multicol;
        this.placeholder = field.placeholder;
        this.multicol = field.multicol === undefined ? 0 : field.multicol;
        this.help = field.help === undefined ? undefined : new Help(field.help);
        this.choices = field.choices === undefined ? [] : field.choices;
        this.params = field.params === undefined ? [] : field.params;
        this.required = field.required === undefined ? false : true;
        this.defaultValue = field.defaultValue === undefined ? undefined : field.defaultValue;
        this.dynamicValue = field.dynamicValue === undefined ? undefined : field.dynamicValue;
        this.validations = field.validations === undefined ? [] : field.validations;
        this.conditions = field.conditions === undefined ? [] : field.conditions;
        this.addClass('field-' + this.type);
        this.addClass('mb-3');
        field.classes?.forEach(className => this.addClass(className));
        field.fields?.forEach(field => this.fields.push(new Field(this.fieldset, field)));
        this.value = field.value;
    }
    /**
     * Set value of field.
     *
     * @param value Value to set.
     *
     * @since 1.0.0
     */
    setValue(value) {
        this.value = value;
    }
    /**
     * Get value of field.
     *
     * @return value Value to set.
     *
     * @since 1.0.0
     */
    getValue() {
        return this.value;
    }
    /**
     * Is there a prefix?
     *
     * @returns True if field has prefix.
     *
     * @since 1.0.0
     */
    hasPrefix() {
        return this.prefix !== undefined;
    }
    /**
     * Get prefix.
     *
     * @returns Prefix.
     *
     * @since 1.0.0
     */
    getPrefix() {
        return this.prefix;
    }
    /**
     * Get suffix.
     *
     * @returns Suffix.
     *
     * @since 1.0.0
     */
    getSuffix() {
        return this.prefix;
    }
    /**
     * Is there a suffix?
     *
     * @returns True if field has suffix.
     *
     * @since 1.0.0
     */
    hasSuffix() {
        return this.prefix !== undefined;
    }
    /**
     * Get default value.
     *
     * @return value Default value.
     *
     * @since 1.0.0
     */
    getDefaultValue() {
        if (this.defaultValue === undefined) {
            return;
        }
        let dynamicValue = new DynamicValue(this.fieldset.form, this.defaultValue);
        return dynamicValue.getValue();
    }
    /**
     * Get default value.
     *
     * @return value Default value.
     *
     * @since 1.0.0
     */
    getDynamicValue() {
        if (this.dynamicValue === undefined) {
            return;
        }
        let dynamicValue = new DynamicValue(this.fieldset.form, this.dynamicValue);
        return dynamicValue.getValue();
    }
    /**
     * Sets autovalues for default or dynamic value if exists.
     *
     * @since 1.0.0
     */
    autoValue() {
        if (this.value == undefined) {
            this.value = this.getDefaultValue();
        }
        let dynamicValue = this.getDynamicValue();
        if (dynamicValue !== undefined) {
            this.value = dynamicValue;
        }
    }
    /**
     * Does field have choices.
     *
     * @return True if it has choices, false if not.
     *
     * @since 1.0.0
     */
    hasChoices() {
        return this.choices.length > 0;
    }
    /**
     * Validate the field.
     *
     * @return Array of errors, empty array on no errors.
     *
     * @since 1.0.0
     */
    validate() {
        if (!this.conditionsFullfilled())
            return [];
        let validator = new Validator(this.value, this.validations);
        this.errors = validator.check();
        if (this.errors.length > 0) {
            this.addClass('is-invalid');
            this.removeClass('is-valid');
        }
        else {
            this.addClass('is-valid');
            this.removeClass('is-invalid');
        }
        this.validated = true;
        return this.errors;
    }
    /**
     * Get validation errors.
     *
     * @return Erros which occured while validating.
     *
     * @since 1.0.0
     */
    getValidationErors() {
        return this.errors;
    }
    /**
     * Has field validation errors.
     *
     * @return True if field has errors, false if not.
     *
     * @since 1.0.0
     */
    hasValidationErrors() {
        if (this.errors.length > 0) {
            return true;
        }
        return false;
    }
    /**
     * Has the field already ben validated
     *
     * @return True if field has been validated, false if not.
     *
     * @since 1.0.0
     */
    hasBeenValidated() {
        return this.validated;
    }
    /**
     * Checks if conditions are fullfilled.
     *
     * @returns True if conditions are fullfilled, false if not.
     *
     * @since 1.0.0
     */
    conditionsFullfilled() {
        if (this.conditions.length === 0) {
            return true;
        }
        let fullfillments = [];
        this.conditions.forEach((condition) => {
            let fullfilled = false;
            let field = this.fieldset.form.getField(condition.field);
            switch (condition.operator) {
                case '==':
                    fullfilled = condition.value === field.getValue();
                    break;
                case '!=':
                    fullfilled = condition.value !== field.getValue();
                    break;
                case '>':
                    fullfilled = condition.value !== field.getValue();
                    break;
                case '<':
                    fullfilled = condition.value !== field.getValue();
                    break;
                default:
                    throw new Error('Operator "' + condition.operator + '" does not exist.');
            }
            fullfillments.push(fullfilled);
        });
        return !fullfillments.includes(false);
    }
    /**
     * Add input class.
     *
     * @param string Name of css class to add.
     *
     * @since 1.0.0
     */
    addInputClass(name) {
        if (this.inputClasses.indexOf(name) !== -1) {
            return;
        }
        this.inputClasses.push(name);
    }
    /**
     * Remove input class.
     *
     * @param string Name of css class to remove.
     *
     * @since 1.0.0
     */
    removeInputClass(name) {
        this.inputClasses = this.inputClasses.filter(className => className !== name);
    }
    /**
     * Get input classes.
     *
     * @return String of CSS classes.
     *
     * @since 1.0.0
     */
    getInputClasses() {
        if (!this.hasBeenValidated()) {
            return this.inputClasses.join(' ');
        }
        if (!this.hasValidationErrors()) {
            this.addInputClass('is-valid');
            this.removeInputClass('is-invalid');
        }
        else {
            this.addInputClass('is-invalid');
            this.removeInputClass('is-valid');
        }
        return this.inputClasses.join(' ');
    }
}
