import Field from './Field';
import CSSElement from './CSSElement';
/**
 * Class Fieldset.
 *
 * @since 1.0.0
 */
export default class Fieldset extends CSSElement {
    form;
    name;
    label;
    percentage;
    params;
    conditions;
    submission;
    nextFieldset;
    prevFieldset;
    fields = [];
    /**
     * @param fieldset Fieldset data.
     * Initializing fieldset.
     *
     * @param form Form object.
     *
     * @since 1.0.0
     */
    constructor(form, fieldset) {
        super();
        this.form = form;
        this.name = fieldset.name;
        this.label = fieldset.label;
        this.percentage = fieldset.percentage;
        this.params = undefined === fieldset.params ? [] : fieldset.params;
        this.conditions = undefined === fieldset.conditions ? [] : fieldset.conditions;
        this.submission = fieldset.submission;
        this.nextFieldset = fieldset.nextFieldset;
        this.prevFieldset = fieldset.prevFieldset;
        this.addClass('fieldset-' + this.name);
        fieldset.classes?.forEach(className => this.addClass(className));
        fieldset.fields.forEach(field => this.fields.push(new Field(this, field)));
    }
    /**
     * Checks if conditions are fullfilled.
     *
     * @returns True if conditions are fullfilled, false if not.
     *
     * @since 1.0.0
     */
    conditionsFullfilled() {
        if (this.conditions.length === 0) {
            return true;
        }
        let fullfillments = [];
        this.conditions.forEach((condition) => {
            let fullfilled = false;
            let field = this.form.getField(condition.field);
            switch (condition.operator) {
                case '==':
                    fullfilled = condition.value === field.getValue();
                    break;
                case '!=':
                    fullfilled = condition.value !== field.getValue();
                    break;
                case '>':
                    fullfilled = condition.value !== field.getValue();
                    break;
                case '<':
                    fullfilled = condition.value !== field.getValue();
                    break;
                default:
                    throw new Error('Operator "' + condition.operator + '" does not exist.');
            }
            fullfillments.push(fullfilled);
        });
        return !fullfillments.includes(false);
    }
    /**
     * Get a specific field.
     *
     * @param name Name of field.
     * @return Field object. Undefined if nothing was found.
     *
     * @since 1.0.0
     */
    getField(name) {
        let foundField;
        this.fields.forEach((field) => {
            if (field.type === 'group') {
                field.fields.forEach((field) => {
                    if (field.name === name) {
                        foundField = field;
                    }
                });
            }
            else if (field.name === name) {
                foundField = field;
            }
        });
        return foundField;
    }
    /**
     * Validate fieldset.
     *
     * @return True on successful validation, false on errors.
     *
     * @since 1.0.0
     */
    validate() {
        let foundError = false;
        this.fields.forEach((field, i) => {
            if (field.type === 'group') {
                field.fields.forEach((field) => {
                    if (field.validate().length > 0 && !foundError) {
                        foundError = true;
                    }
                });
            }
            else if (field.validate().length > 0 && !foundError) {
                foundError = true;
            }
        });
        return foundError;
    }
    /**
     * Has fieldset validation errors.
     *
     * @return True if fieldset has errors, false if not.
     *
     * @since 1.0.0
     */
    hasValidationErrors() {
        let foundError = false;
        this.fields.forEach((field) => {
            if (field.type === 'group') {
                field.fields.forEach((field) => {
                    if (field.hasValidationErrors() && !foundError) {
                        foundError = true;
                    }
                });
            }
            else if (field.hasValidationErrors() && !foundError) {
                foundError = true;
            }
        });
        return foundError;
    }
}
